DifficultyCacheConfig

Struct DifficultyCacheConfig 

pub struct DifficultyCacheConfig {
    pub window: usize,
    pub cut: usize,
    pub lag: usize,
    pub fixed_difficulty: Option<u128>,
}
Configuration for the difficulty cache.

Fields

window: usizecut: usizelag: usizefixed_difficulty: Option<u128>

If Some the difficulty cache will always return this value as the current difficulty.

Implementations

Source

impl DifficultyCacheConfig

pub const fn total_block_count(&self) -> usize

Returns the total amount of blocks we need to track to calculate difficulty

pub const fn accounted_window_len(&self) -> usize

The amount of blocks we account for after removing the outliers.

pub const fn main_net() -> Self

Returns the config needed for Mainnet. This is also the config for all other current networks.