pub struct DifficultyCacheConfig {
pub window: usize,
pub cut: usize,
pub lag: usize,
pub fixed_difficulty: Option<u128>,
}Expand description
Configuration for the difficulty cache.
Fields
window: usizecut: usizelag: usizefixed_difficulty: Option<u128>If Some the difficulty cache will always return this value as the current difficulty.
Implementations
Sourceimpl DifficultyCacheConfig
impl DifficultyCacheConfig
Sourcepub const fn total_block_count(&self) -> usize
pub const fn total_block_count(&self) -> usize
Returns the total amount of blocks we need to track to calculate difficulty
Sourcepub const fn accounted_window_len(&self) -> usize
pub const fn accounted_window_len(&self) -> usize
The amount of blocks we account for after removing the outliers.